import os
from PIL import Image,ImageDraw,ImageFont

from skeletonz.modules.plugin import GenericPlugin
from skeletonz.user_plugins import TextOption
from skeletonz.server import getFormatManager

PLUGINS_FOR_EXPORT = ['ImgTitle']

class ImgTitle(GenericPlugin):
    NAME = "Image Title plugin"
    DESCRIPTION = "Generate images from fonts and text.<br />Python Image Library (PIL) required."
    SYNTAX = [
        {'handler': 'imgtitle',
         'required_arguments': {'ident':'Text to be converted'},
         'optional_arguments': {'font':'Font to be used', 'size':'Size to be used'}
        },
      ]
    PLUGIN_OPTIONS = [
            TextOption(scope='ImgTitle',name="font_path",text="Font path",default="/usr/share/fonts/truetype/freefont"),
            TextOption(scope='ImgTitle',name="default_font",text="Default font",default="FreeSans.ttf"),
            TextOption(scope='ImgTitle',name="default_size",text="Default size",default="16")
                      ]
    def __init__(self):
        getFormatManager().registerSLPlugin('imgtitle', self.generateImage)

    def toFileName(self,text,font,size):
        filename = "%s_%s_%s.png" % (text, font, str(size))
        return filename.replace(' ','_')

    def generateImage(self,args,edit_mode,page_id):
        title = args.get('imgtitle', "No title provided !")

        # Check defaults and args and load the requested font if the image does not already exists
        font = args.get('font', self.optionValue('default_font'))
        size = int(args.get('size', self.optionValue('default_size')))
        font_file = os.path.join(self.optionValue('font_path'),font)

        img_file = self.toFileName(title,font,size)
        if not os.path.isfile(os.path.join(self.img_folder,img_file)):
            imgfont = ImageFont.truetype(font_file, size)
            # Dummy image creation to size the image, then real image generation
            img = Image.new("RGBA",(500,500),(255,255,255))
            draw = ImageDraw.Draw(img)
            width,height = draw.textsize(title, font=imgfont)
            img = img.resize((width,height))
            draw = ImageDraw.Draw(img)
            draw.text((0,0), title,fill=(0,0,0),font=imgfont)
            img.save(os.path.join(self.img_folder,img_file))

        return False, "<img src='/generated/imgtitle/%s'/>" % img_file

    def createStructure(self):
        self.img_folder = 'dynamic_dirs/generated/imgtitle'
        if not os.path.isdir(self.img_folder):
            os.mkdir(self.img_folder)
        return []
