import os
import md5

from skeletonz.modules.plugin import GenericPlugin
from skeletonz.user_plugins import ListOption, TextOption
from skeletonz.server import getFormatManager

PLUGINS_FOR_EXPORT = ['LatexPlugin']

class LatexPlugin(GenericPlugin):
    NAME = "LaTeX plugin"
    DESCRIPTION = "Display LaTeX equations on your site.<br />LaTeX and ImageMagic required."
    SYNTAX = [
        {'handler': 'ltx',
         'required_arguments': {'ident': 'LaTeX formula'},
         'optional_arguments': {}
        },
        {'handler': 'latex',
          'required_arguments': {'data': {'type': 'text', 'help': 'LaTeX formula'}},
         'optional_arguments': {}
        }
      ]

    PLUGIN_OPTIONS = [
      TextOption(scope='Latex', name="resolution", text="Resolution", default="90x90")
    ]

    def __init__(self):
        getFormatManager().registerSLPlugin('ltx', self.handle)
        getFormatManager().registerMLPlugin('latex', self.handle)

    def handle(self, args, edit_mode, page):
        latex = None
        if args.get('data', False):
            latex = args['data']

        if args.get('ltx', False):
            latex = args['ltx']

        resolution = self.optionValue("resolution")

        if latex:
            latex = latex.replace("\[", "[").replace("\]", "]")

            #Generate unique name
            uniq = hash(latex)
            #Check if the file is already generated
            full_path = 'site_plugins/latex/static/%s_%s.png' % (resolution, uniq)

            try:
                open(full_path)
            except:
                write_file = "site_plugins/latex/tmp_%s.tex" % uniq
                open(write_file, "w").write(latex)
                gen = "sh site_plugins/latex/tex2im -r%s -o%s %s"
                command = gen % (resolution, full_path, write_file)
                os.popen(command)
                os.remove(write_file)
            return False, '<img src="static_plugin/latex/%s_%s.png" align="bottom" />' % (resolution, uniq)

    def addToController(self, rc):
        path = "%s/site_plugins/latex/static" % os.getcwd()
        rc.addStaticPath("/static_plugin/latex/", path)
