import os
import re
import datetime
import shelve

from amilib.useful import Singleton
from amilib.amiweb import amiweb

from skeletonz.modules.plugin import GenericPlugin
from skeletonz.mylib import html_helpers
from amilib.template import render
from skeletonz.modules.template import PluginTemplate

from skeletonz.server import getConfig, getFormatManager

from skeletonz import Site
from skeletonz.Site import adminPermission, editPermission

from skeletonz.mylib.amicache import AmiCache

PLUGINS_FOR_EXPORT = ['MailinglistManagerPlugin']


class MailinglistManagerPlugin(GenericPlugin):
    NAME = "Mailing list manager"
    DESCRIPTION = "Mailing list subscribe and manage."
    SYNTAX = [
        {'handler': 'ml_subscribe',
         'required_arguments': {
           'ident': 'The text',
           'file': 'The full path to the file that emails should be added to'},
         'optional_arguments': {}
        },

        {'handler': 'ml_manage',
         'required_arguments': {
           'ident': 'The text',
           'files': 'The full path to mailing list files separated by ,'},
         'optional_arguments': {}
        },

      ]

    def __init__(self):
        self.sub_controller = SubscribeController()
        getFormatManager().registerSLPlugin('ml_subscribe', self.handleSubscribe)
        getFormatManager().registerSLPlugin('ml_manage', self.handleManage)

    def handleSubscribe(self, args, edit_mode, page):
        if not args.has_key("ml_subscribe") or not args.has_key("file"):
            return None

        key = hash(args['file'])

        add_link = "%s/plugin/mailinglist_manager/viewAddEmail?key=%s" % (getConfig().BASE_URL, key)
        js = "return GB_show('Add your email', '%s', 250, 400);" % add_link

        return False, """<a href="%s" onclick="%s">%s</a>""" % (add_link, js, args['ml_subscribe'])

    def handleManage(self, args, edit_mode, page):
        files = re.split(r'\s*\|\s*', args['files'])
        self.sub_controller.files = files
        self.sub_controller.path = args['path']

        manage_link = "%s/plugin/mailinglist_manager/viewManageEmail" % (getConfig().BASE_URL)

        args['manage_link'] = "return GB_showFullScreen('Manage announcement list', '%s')" % manage_link
        return False, '<a href="" onclick="%(manage_link)s">%(ml_manage)s</a>' % args

    def addToController(self, rc):
        path = "%s/site_plugins/mailinglist_manager/static/" % os.getcwd()
        rc.addStaticPath("/static_plugin/mailinglist/", path)

        rc.root_obj.plugin.mailinglist_manager = self.sub_controller


class SubscribeController(Singleton):

    path = ''

    def __init__(self):
        self.database = shelve.open('ml_database.dat', writeback=True)

        template = PluginTemplate("mailinglist_manager")
        self.header = template.getHeader()
        self.footer = template.getFooter()

        self.files = {}

    def getFileByKey(self, key):
        for file in self.files:
            if hash(file) == key:
                return file

    def getEmailsByKey(self, key):
        file = self.getFileByKey(key)
        if file:
            return self.files[file]
        return None

    def saveEmails(self):
        self.database.sync()
        for file in self.files:
            emails = []
            for email in self.database:
                item = self.database[email]
                if item['added_to'].has_key(file):
                    emails.append(email)

            self._saveEmails(file, emails)

    def _saveEmails(self, file, emails):
        fp = open('%s%s' % (self.path, file), "w")
        emails = ['%s,\n' % email for email in emails]
        fp.writelines(emails)
        fp.close()

    @amiweb.expose
    def viewAddEmail(self, key):
        ns = {'site': self,
              'key': key}
        return render("site_plugins/mailinglist_manager/view/add.tmpl", ns)

    @amiweb.expose
    @editPermission
    def viewRemoveEmail(self, key):
        ns = {'site': self,
              'key': key}
        return render("site_plugins/mailinglist_manager/view/remove.tmpl", ns)

    @amiweb.expose
    @editPermission
    def viewManageEmail(self):
        ns = {'site': self,
              'database': self.database,
              'all_files': self.files,
              'renderItems': self.renderItems}
        return render("site_plugins/mailinglist_manager/view/manage.tmpl", ns)

    @amiweb.expose
    def deleteEmailAnnounce(self, email, key):
        key = int(key)
        file = self.getFileByKey(key)

        if self.database.has_key(email):
            try:
                del self.database[email]['added_to'][file]
            except:
                return 'error'
        else:
            return "error"

        self.saveEmails()
        return "ok"

    @amiweb.expose
    @editPermission
    def deleteEmail(self, email):
        try:
            del self.database[email]
            self.saveEmails()
        except:
            pass
        return 'ok'

    @amiweb.expose
    @editPermission
    def removeSubscribsion(self, email, key):
        key = int(key)
        try:
            file = self.getFileByKey(key)
            del self.database[email]['added_to'][file]
            self.saveEmails()
        except ValueError:
            return "error"

        return self.renderItems()

    @amiweb.expose
    @editPermission
    def editEmail(self, old_email, new_email, new_comment):
        if old_email != new_email:
            self.database[new_email] = self.database[old_email]
            del self.database[old_email]
        self.database[new_email]['comment'] = new_comment

    def renderItems(self):
        ns = {
            'all_files': self.files,
            'database': self.database,
            'renderItem': self.renderItem
        }

        return render("site_plugins/mailinglist_manager/view/items.tmpl", ns)

    def renderItem(self, email, item, new_item=False):
        def getDeleteBtn(email):
            return html_helpers.createActionLink("", "static_plugin/mailinglist/delete.png", "return deleteEmail('%s', this)" % (email), 'CMS_link CMS_critical_link')

        def getEditBtn(email, comment):
            return html_helpers.createActionLink("", "static_plugin/mailinglist/manage.png", "return editEmail('%s', '%s', this)" % (email, comment.replace("'", "\\'")), 'CMS_link')

        ns = {
            'all_files': self.files,
            'getDeleteBtn': getDeleteBtn,
            'getEditBtn': getEditBtn,
            'email': email,
            'item': item,
            'new_item': new_item
        }
        return render("site_plugins/mailinglist_manager/view/item.tmpl", ns)

    @amiweb.expose
    def addEmailInternal(self, comment, email):
        self.database[email] = {'comment': comment, 'added_to': {}}
        self.saveEmails()
        return self.renderItem(email, self.database[email], True)

    @amiweb.expose
    def addEmail(self, email, key):
        key = int(key)
        email = email.strip()

        file = self.getFileByKey(key)

        self.database[email]['added_to'][file] = True

        self.saveEmails()

        ns = {'site': self,
              'email': email}
        return render("site_plugins/mailinglist_manager/view/added.tmpl", ns)

    @amiweb.expose
    def addEmailExternal(self, email, name, key):
        key = int(key)
        email = email.strip()
        file = self.getFileByKey(key)

        if not self.database.has_key(email):
            self.database[email] = {'comment': name, 'added_to': {file: True}}
        else:
            self.database[email]['added_to'][file] = True
        print self.database[email]

        self.saveEmails()

        ns = {'site': self,
              'email': email}
        return render("site_plugins/mailinglist_manager/view/added.tmpl", ns)
