import os
import re
from datetime import datetime

from amilib.amiweb.amiweb import db, session
from amilib.amiweb import amiweb
from amilib import PyRSS2Gen

from skeletonz.model.CMSModel import Pages

from skeletonz.mylib.amicache import AmiCache

from skeletonz.mylib import html_helpers, converters
from amilib.template import render
from skeletonz.modules.plugin import GenericPlugin
from skeletonz.server import getConfig, getRSSManager, plugin_configurator, getFormatManager
from skeletonz.modules.template import PluginTemplate

from amilib.amiweb.amigration import *

from skeletonz.Site import adminPermission, editPermission

PLUGINS_FOR_EXPORT = ['NewsSectionPlugin']

class NewsSectionPlugin(GenericPlugin):
    NAME = "News section"
    DESCRIPTION = "Adds a news manager that can output RSS."
    SYNTAX = [
        {'handler': 'news_section',
         'required_arguments': {'ident': 'The identification'},
         'optional_arguments': {'limit': 'Set how many news items should be showed',
                                'rss_icon': {'type': 'option', 'help': 'Should the rss icon be shown?'},
                                'order': {'help': 'Can be <b>date</b> or <b>manual</b>', 'default': 'date'},
                                'status': {'help': 'Is used to view past and future items. Can be <b>past</b>, <b>current</b>, <b>future</b> or <b>all</b>', 'default': 'current'}}
        }
      ]

    def __init__(self):
        self.mi_control = AMigrationControl("NewsSection",\
            [NewsSection_Initial, NewsSection_AddIdent, NewsSection_AddValidity, NewsSection_AddOrder], plugin_configurator)
        getFormatManager().registerSLPlugin('news_section', self.handleFilter)

    def addToSiteEditTemplate(self, template, on_init):
        news_section_script = render("site_plugins/news_section/view/script.tmpl", {})
        template.getHeader().appendStaticScriptData(news_section_script)

    def handleFilter(self, args, edit_mode, page):
        r = NewsSectionController.getHTMLData(args['news_section'], edit_mode, args.get("limit", 5), args.get('rss_icon', 'on'), args.get('order', 'date'), args.get('status', 'current'))
        return True, r

    def addToController(self, rc):
        path = "%s/site_plugins/news_section/static" % os.getcwd()
        rc.addStaticPath("/static_plugin/news_section/", path)

        rc.root_obj.plugin.news_section = NewsSectionController

    def createStructure(self):
        self.mi_control.upgradeToLatest()

    def dropStructure(self):
        self.mi_control.downgradeTo(Not_Created)


##
# Controller
#
class NewsSectionController:

    def __init__(self):
        template = PluginTemplate("news_section")
        self.header = template.getHeader()
        self.header.appendEditorScripts()
        self.footer = template.getFooter()

    @amiweb.expose
    @editPermission
    def viewAdd(self, ident, status):
        ns = {'site': self,
              'ident': ident,
              'status': status}
        return render("site_plugins/news_section/view/add_item.tmpl", ns)

    @amiweb.expose
    @editPermission
    def viewEdit(self, id):
        item = NewsSectionModel.getItemById(id)
        ns = {'site': self,
              'id': id,
              'current_item': item}
        return render("site_plugins/news_section/view/edit_item.tmpl", ns)

    @amiweb.expose
    def getHTMLData(self, ident, edit_mode=False, limit=5, rss_icon='on', order='date', status='current'):
        if edit_mode == "True":
            edit_mode = True
        if edit_mode == "False":
            edit_mode = False

        btn_add_news = html_helpers.createActionLink("Add item", "static_plugin/news_section/add.png", "return NewsSection.viewAdd('%s', this, '%s');" % (ident, status))

        if order == 'manual':
            orderby = 'n_order'
        else:
            orderby = 'valid_to, id'

        news_items = db().select("plugin_news_section", ident=ident, order_by=orderby, reversed=True, limit=limit)

        if edit_mode:
            btn_delete = html_helpers.createActionLink("Delete item", "static_plugin/news_section/delete.png", "NewsSection.del(this)", "CMS_link CMS_critical_link", tooltip_inner=True)
            btn_edit = html_helpers.createActionLink("Edit item", "static_plugin/news_section/edit.png", "NewsSection.viewEdit(this)", tooltip_inner=True)
            if order == 'manual':
                btn_up = html_helpers.createActionLink("Move item up", "static_plugin/news_section/uparrow.gif", "NewsSection.move(this, 'Up')", tooltip_inner=True)
                btn_down = html_helpers.createActionLink("Move item down", "static_plugin/news_section/downarrow.gif", "NewsSection.move(this, 'Down')", tooltip_inner=True)
            else:
                btn_up = ""
                btn_down = ""
        else:
            btn_edit = ""
            btn_delete = ""
            btn_up = ""
            btn_down = ""

        news_items = NewsSectionModel.filterActiveItems(news_items, status, edit_mode)

        BASE_URL = getConfig().BASE_URL
        rss_ident = "news_section_%s.xml" % converters.makeLinkAble(ident)

        page_obj = Pages.getPageById(session()['current_page_id'])

        def filterContent(text):
            return getFormatManager().htmlFormat(text, edit_mode, False, page_obj)

        ns = {'ident': ident,
              'news_items': news_items,
              'filterContent': filterContent,
              'edit_mode': edit_mode,
              'btn_delete': btn_delete,
              'btn_edit': btn_edit,
              'btn_up': btn_up,
              'btn_down': btn_down,
              'limit': limit,
              'rss_icon': rss_icon,
              'order': order,
              'status': status,
              'RSS_feed': '%s/rss/%s' % (BASE_URL, rss_ident),
              'btn_add_news': btn_add_news}
        return render("site_plugins/news_section/view/items.tmpl", ns)

    @amiweb.expose
    @editPermission
    def add(self, ident, n_content, valid_from, valid_to):
        AmiCache.expireCurrentPage()
        NewsSectionModel.add(ident, n_content, valid_from, valid_to)
        ns = {'site': self}
        return render("site_plugins/news_section/view/item_updated.tmpl", ns)

    @amiweb.expose
    @editPermission
    def update(self, id, new_content, valid_from, valid_to, force_activation):
        AmiCache.expireCurrentPage()
        NewsSectionModel.update(id, new_content, valid_from, valid_to, force_activation)
        ns = {'site': self}
        return render("site_plugins/news_section/view/item_updated.tmpl", ns)

    @amiweb.expose
    @editPermission
    def delete(self, id):
        AmiCache.expireCurrentPage()
        NewsSectionModel.delete(id)
        return "ok"

    @amiweb.expose
    @editPermission
    def moveUp(self, id):
        AmiCache.expireCurrentPage()
        NewsSectionModel.moveUp(id)
        return "ok"

    @amiweb.expose
    @editPermission
    def moveDown(self, id):
        AmiCache.expireCurrentPage()
        NewsSectionModel.moveDown(id)
        return "ok"

##
# Model
#
class NewsSectionModel:

    def getItemById(self, id):
        return db().select("plugin_news_section", id=id, as_one=True)

    def add(self, ident, n_content, valid_from, valid_to):
        n_order = db().selectMax('plugin_news_section', 'n_order')
        if n_order == None:
            n_order = 0
        else:
            n_order += 1

        db().insert("plugin_news_section", ident=ident, n_content=n_content, valid_from=valid_from, valid_to=valid_to, n_order=n_order)
        self.updateXML(ident)

    def update(self, id, new_content, valid_from, valid_to, force_activation):
        ident = self.getItemById(id).ident
        db().update("plugin_news_section", id=id, n_content=new_content, valid_from=valid_from, valid_to=valid_to, force_activation=force_activation)
        self.updateXML(ident)

    def delete(self, id):
        ident = self.getItemById(id).ident
        db().delete("plugin_news_section", id=id)
        self.updateXML(ident)

    def moveUp(self, id):
        ident = self.getItemById(id).ident
        n_order = db().select("plugin_news_section", id=id, as_one=True).n_order
        above = db().select("plugin_news_section", ident=ident, where="n_order > %s" % (n_order), order_by="n_order", reversed=False, as_one=True)
        if above:
            db().update("plugin_news_section", id=id, n_order=above.n_order)
            db().update("plugin_news_section", id=above.id, n_order=n_order)
            self.updateXML(ident)

    def moveDown(self, id):
        ident = self.getItemById(id).ident
        n_order = db().select("plugin_news_section", id=id, as_one=True).n_order
        below = db().select("plugin_news_section", ident=ident, where="n_order < %s" % (n_order), order_by="n_order", as_one=True, reversed=True)
        if below:
            db().update("plugin_news_section", id=id, n_order=below.n_order)
            db().update("plugin_news_section", id=below.id, n_order=n_order)
            self.updateXML(ident)

    def updateXML(self, ident):
        rss_file = "news_section_%s.xml" % converters.makeLinkAble(ident)
        rss_link = "%s%s" % (getConfig().BASE_URL, "/rss/%s" % rss_file)
        rss_items = []
        db_items = db().select("plugin_news_section", ident=ident, limit=10, order_by="valid_to, id", reversed=True)

        page_obj = Pages.getPageById(session()['current_page_id'])
        def filterContent(text):
            return getFormatManager().htmlFormat(text, False, False, page_obj)

        def minifiy(text):
            text = re.sub(r'<.*?>(.*?)</.*?>', r'\1', text)
            if len(text) < 50:
                return text
            result = text[0:50]
            f_c = result.find('\n')
            if f_c == -1:
                f_c = result.find('.')
            if f_c == -1:
                f_c = result.find('<')
            if f_c != -1 and f_c != 0:
                return result[0:f_c].strip()
            return "%s..." % result.strip()

        for item in self.filterActiveItems(db_items):
            item_content = filterContent(item.n_content)
            rss_items.append(
             PyRSS2Gen.RSSItem(
               title = minifiy(item_content),
               description = "%s" % item_content
             )
            )

        rss_obj = PyRSS2Gen.RSS2(
          title = ident,
          link = rss_link,
          description = "",
          lastBuildDate = datetime.now(),
          items = rss_items
        )

        getRSSManager().publish(rss_file, rss_obj)

    def filterActiveItems(self, items, status='current', edit_mode=True):
        # Removes inactive news from published the list
        return [elt for elt in items if elt['force_activation']=='active' or
               (elt['force_activation']=='inactive' and edit_mode) or
               (elt['force_activation']=='off' and (elt['valid_to']>datetime.now() and 
                elt['valid_from']<datetime.now() and status=='current' or elt['valid_from']>datetime.now() and status =='future' or elt['valid_to']<datetime.now() and status =='past' or status=='all'))]


class NewsSection_Initial(AmiGration):

    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.createTable("plugin_news_section",
          IntCol("id", length=11, auto_increment=True),
          StringCol("n_content", not_null=True),
          PrimaryKey("id"),
          utf8=True,
          ignore_if_created=True
        )

    def down(self):
        self.dropTable("plugin_news_section")


class NewsSection_AddIdent(AmiGration):

    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.addColumn("plugin_news_section", StringCol("ident", length=100, not_null=True))
        UniqueKey("plugin_news_section", "ident")

    def down(self):
        self.removeColumn("plugin_news_section", "ident")


class NewsSection_AddValidity(AmiGration):

    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.addColumn("plugin_news_section", DateTimeCol("valid_from", not_null=True, default='1900-01-01'))
        self.addColumn("plugin_news_section", DateTimeCol("valid_to", not_null=True, default='2100-12-31'))
        self.addColumn("plugin_news_section", StringCol("force_activation", length=8, not_null=True, default='off')) #off / active / inactive

    def down(self):
        self.removeColumn("plugin_news_section", "valid_from")
        self.removeColumn("plugin_news_section", "valid_to")
        self.removeColumn("plugin_news_section", "force_activation")

class NewsSection_AddOrder(AmiGration):

    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.addColumn("plugin_news_section", IntCol("n_order", not_null=True, default='0'))

        news_items = db().select("plugin_news_section", order_by='valid_to, id', reversed=True)
        n_order = 1
        for item in news_items:
            id = item.id
            db().update("plugin_news_section", id=id, n_order=n_order)
            n_order = n_order + 1

    def down(self):
        self.removeColumn("plugin_news_section", "n_order")

NewsSectionController = NewsSectionController()
NewsSectionModel = NewsSectionModel()
