import re
import os
import types
from amilib import json

from amilib.amiweb.amiweb import db, session

from skeletonz.mylib.converters import makeLinkAble
import skeletonz.plugins as plugins
from skeletonz.plugins.upload import filter as upload_filter
from skeletonz.server import getConfig, getFormatManager
from amilib.template import render

import skeletonz.model.CMSModel as Model

class Table:

    def getAllItems(self):
        return db().select("plugin_personnel_item", table_id=self.id, order_by="item_order")

    def getName(self):
        name = self.name
        for c in (" ", ".", "_"):
            name = name.replace(c, "_")
        return name

    def getHostPage(self):
        return Model.Pages.getPageById(self.host_page)


class Tables:

    def getTableByName(self, name):
        item = db().select("plugin_personnel_table", name=name, as_one=True, obj_deco=Table)
        if item == None:
            return self.insertTable(name)
        else:
            return item

    def updateHostPage(self, name):
        current_page_id = session()['current_page_id']
        db().update('plugin_personnel_table', where='name="%s"' % name, host_page=current_page_id)

    def getTableById(self, id):
        return db().select("plugin_personnel_table", id=id, as_one=True, obj_deco=Table)

    def insertTable(self, name):
        n = db().insert("plugin_personnel_table", name=name)
        self.updateHostPage(name)
        return self.getTableById(n)

    def resetTable(self, id):
        #Delete all items
        db().delete("plugin_personnel_item", table_id=id)

    def sortTable(self, id):
        table = self.getTableById(id)
        items = table.getAllItems()

        def cmp(a, b):
            a_lname = json.read(a.json_data)['Name'].split(" ")[-1]
            b_lname = json.read(b.json_data)['Name'].split(" ")[-1]
            if a_lname > b_lname:
                return 1
            elif a_lname == b_lname:
                return 0
            else:
                return -1

        items.sort(cmp)

        order = 0
        for item in items:
            TableItems.setNewOrder(item.id, order)
            order += 1


class TableItems:

    def deleteItem(self, id):
        db().delete("plugin_personnel_item", id=id, as_one=True)

    def getItemById(self, id):
        return db().select("plugin_personnel_item", id=id, as_one=True)

    def getItemByName(self, name):
        if type(name) == type(''):
            name = unicode(name, 'utf8')
        name = name.encode('utf8')
        like = "".join(['%"Name":"', db().escape(name), '"%'])
        return db().select("plugin_personnel_item", where="json_data LIKE '%s'" % like, as_one=True)

    def updateData(self, id, new_json):
        db().update("plugin_personnel_item", id=id, json_data=new_json)

    def setNewOrder(self, id, new_order):
        db().update('plugin_personnel_item', id=id, item_order=new_order)

    def insertItem(self, table_id, json_data):
        item_order = db().selectMax('plugin_personnel_item', 'item_order', table_id=table_id)
        if item_order == None:
            item_order = 0
        else:
            item_order += 1
        n = db().insert("plugin_personnel_item", table_id=table_id, json_data=json_data, item_order=item_order)
        return self.getItemById(n)

    def getEditView(self, item_id):
        elm = self.getItemById(item_id)
        return self.generateEditHTML(json.read(elm.json_data), item_id)

    def generateEditHTML(self, data, item_id):
        attrs = "this, '%s'" % (item_id)
        control_buttons = '<span class="CMS_link CMS_critical_link" onclick="Personnel.saveData(%s)"><img src="%s/static_core/images/save.png" alt="Save" />Save changes</span>' % (attrs, getConfig().BASE_URL)

        image = getFormatManager().htmlFormat('[personnelimage=person_%s]' % item_id, True, False)

        data['image'] = image
        data['NameLink'] = makeLinkAble(data['Name'])
        data['ItemId'] = item_id
        data['ControlButtons'] = control_buttons

        template_text = """<div class="Person" id="person_$ItemId">
      <form style="padding: 0; margin: 0;" onsubmit="return false">
      <h3><input type="text" value="$Name" class="Name" /> $ControlButtons</h3>
      <table>
      <tr valign="top">
        <td class="person_image">$image</td>
        <td>
        <table>
        <tr valign="top">
          <td class="person_data">Position:</td>
          <td><input type="text" value="$Position" class="Position" /></td>
        </tr>
        <tr valign="top" class="person_highlight">
          <td>Field:</td>
          <td><input type="text" value="$Field" class="Field" /></td>
        </tr>
        <tr valign="top">
          <td>Office:</td>
          <td><input type="text" value="$Office" class="Office" /></td>
        </tr>

        <tr valign="top" class="person_highlight">
          <td>Phone:</td>
          <td><input type="text" value="$Phone" class="Phone" /></td>
        </tr>
        <tr valign="top">
          <td>E-mail:</td>
          <td><input type="text" value="$Email" class="Email" /></td>
        </tr>
        <tr valign="top" class="person_highlight">
          <td>Home page:</td>
          <td><input type="text" value="$Homepage" class="Homepage" /></td>
        </tr>
        </table>
        </td>
      </tr>
      </table>
      </div>"""
        return render(template_text, ns=data, is_string=True)

    def generateHTML(self, data, item_id, edit_mode):
        try:
            if data.keys() == []:
                #Delete this since its json data is empty!
                self.deleteItem(item_id)
                return ""
        except:
            return ""

        #Set image
        image = getFormatManager().htmlFormat('[personnelimage=person_%s]' % item_id, False, False)

        if edit_mode:
            attrs = "this, '%s'" % (item_id)
            control_buttons = '<span class="CMS_link" onclick="Personnel.editPerson(%s)"><img src="%s/static_plugin/personnel/user_edit.png" alt="Edit data" />Edit data</span>' % (attrs, getConfig().BASE_URL)
            control_buttons += ' <span class="CMS_link CMS_critical_link" onclick="Personnel.deletePerson(%s)"><img src="%s/static_plugin/personnel/user_delete.png" alt="Delete" />Delete</span>' % (attrs, getConfig().BASE_URL)
        else:
            control_buttons = ""

        data['image'] = image
        data['NameLink'] = makeLinkAble(data['Name'])
        data['ItemId'] = item_id
        data['ControlButtons'] = control_buttons

        template_text = """<div class="Person" id="person_$ItemId">
      <a name="$NameLink"></a>
      <h3>$Name $ControlButtons</h3>
      <table>
      <tr valign="top">
        <td class="person_image">$image</td>
        <td>
        <table width="400px">
        <tr valign="top">
          <td class="person_data">Position:</td>
          <td>$Position</td>
        </tr>
        <tr valign="top" class="person_highlight">
          <td>Field:</td>
          <td>$Field</td>
        </tr>
        <tr valign="top">
          <td>Office:</td>
          <td>$Office</td>
        </tr>

        <tr valign="top" class="person_highlight">
          <td>Phone:</td>
          <td>$Phone</td>
        </tr>
        <tr valign="top">
          <td>E-mail:</td>
          <td>$Email</td>
        </tr>
        <tr valign="top" class="person_highlight">
          <td>Home page:</td>
          <td><a href="$Homepage" target="_blank">$Homepage</a></td>
        </tr>
        </table>
        </td>
      </tr>
      </table>
      </div>"""
        return render(template_text, ns=data, is_string=True)

    def insertNewItem(self, table_id):
        json_data = {'Name': '',
               'Position': '',
               'Field': '',
               'Office': '',
               'Phone': '',
               'Email': '',
               'Homepage': ''}
        new_item = self.insertItem(table_id, json.write(json_data))
        html_data = self.generateEditHTML(json_data, new_item.id)
        return html_data


class TableItemsView:

    """
    Mini: Used as real table like
    """
    def generateHTML(self, data, item_id, table_id, color=""):
        name_link = makeLinkAble(data['Name'])
        table = Tables.getTableById(table_id)

        link = "%s#%s" % (table.getHostPage().getFullLink(), name_link)

        link_formated = '<a href="%s">%s</a>' % (link, data["Name"])
        data['link_formated'] = link_formated

        data['color'] = color
        template_text = """
        <tr style="background-color: #%(color)s">
          <td>%(link_formated)s</td>
          <td>%(Position)s</td>
          <td>%(Email)s</td>
          <td>%(Phone)s</td>
          <td>%(Office)s</td>
        </tr>
        """ % data
        return template_text



#Static objects
TableItems = TableItems()
TableItemsView = TableItemsView()
Tables = Tables()
