import os

from amilib.amiweb.amiweb import db, session
from amilib.amiweb import amiweb

#Import from skeletonz
from skeletonz.modules.plugin import GenericPlugin
from skeletonz.model import CMSModel
from skeletonz.model import UserModel
from amilib import json
from skeletonz.mylib.amicache import AmiCache
from amilib.template import render

from skeletonz.server import getConfig, plugin_configurator, getFormatManager

import filter as filter_module
import filter_person_linking
import Tables

from skeletonz import Site

from amilib.amiweb.amigration import *

PLUGINS_FOR_EXPORT = ['PersonnelTable']

class PersonnelTable(GenericPlugin):
    NAME = "Personnel manager"
    DESCRIPTION = "Manage personnel tables with this plugin."
    SYNTAX = [
        {'handler': 'personneltable',
         'required_arguments': {'ident': 'The identification of the table',
                                'type': {'help': 'Can be <b>normal</b> or <b>tableview</b>',
                                         'default': 'normal'}},
         'optional_arguments': {}
        },

        {'handler': 'person',
         'required_arguments': {'ident': 'The name of the person as it stands in the personnel table'},
         'optional_arguments': {'show_as': 'What should the name be displayed as?'},
        }
      ]

    def __init__(self):
        self.prefix = getConfig().TABLE_PREFIX
        self.mi_control = AMigrationControl("PersonnelTablesModel",\
            [PersonnelTablesModel, PersonnelTablesAddHostPage, PersonnelTablesChangeTableNames], plugin_configurator)

        getFormatManager().registerSLPlugin('personneltable', self.handlePersonnelTable)
        getFormatManager().registerSLPlugin('person', self.handlePerson)

    def addToController(self, rc):
        path = "%s/site_plugins/personneltable/static/" % os.getcwd()
        rc.addStaticPath("/static_plugin/personnel/", path)

        rc.root_obj.plugin.personnel_tables = PersonnelTables

    def addToSiteEditTemplate(self, template, on_init):
        if on_init:
            ns = {}
            personnel_script = render("site_plugins/personneltable/view/personnel_table.tmpl", ns)
            template.getHeader().appendStaticScriptData(personnel_script)

    def createStructure(self):
        self.mi_control.upgradeToLatest()

    def dropStructure(self):
        self.mi_control.downgradeTo(Not_Created)


    ##
    # Filter function
    def handlePersonnelTable(self, args, edit_mode, page):
        name = args.get('personneltable')
        if args.has_key("type"):
            if args['type'] == 'normal':
                if edit_mode:
                    Tables.Tables.updateHostPage(name)
                table = Tables.Tables.getTableByName(name)
                return True, filter_module.getPersonelTableHTML(table, edit_mode)
            elif args['type'] == 'tableview':
                return True, filter_module.getPersonelTableHTMLView(name, edit_mode)

    def handlePerson(self, args, edit_mode, page):
        return False, filter_person_linking.wikiPerson(args, page)


##
# Controller
#
class PersonnelTables:

    def filter(self):
        Site.Users.checkCurrentPagePermissions()

    def expirePagesByTableId(self, id):
        table = Tables.Tables.getTableById(id)
        t_name = table.name.encode("utf8")
        pages = CMSModel.Pages.getAllPagesWhereTextIsUsed("\[personneltable=%s,%s\]" % (t_name, '%'))
        for page in pages:
            AmiCache.expireCache(page.id)

    def getTableHTML(self, id):
        table = Tables.Tables.getTableById(id)
        return filter_module.getPersonelTableHTML(table, edit_mode=True)

    @amiweb.expose
    def insertNewItem(self, data):
        data = json.read(data)
        table_id = data['table_id']
        new_item = Tables.TableItems.insertNewItem(table_id)

        self.expirePagesByTableId(table_id)
        return new_item

    @amiweb.expose
    def deleteTable(self, id):
        Tables.Tables.resetTable(id)
        self.expirePagesByTableId(id)
        return self.getTableHTML(id)

    @amiweb.expose
    def sortTable(self, id):
        Tables.Tables.sortTable(id)
        return self.getTableHTML(id)

    @amiweb.expose
    def deletePerson(self, item_id):
        item = Tables.TableItems.getItemById(item_id)
        self.expirePagesByTableId(item.table_id)

        Tables.TableItems.deleteItem(item_id)
        return self.getTableHTML(item.table_id)

    @amiweb.expose
    def saveData(self, item_id, new_json):
        #Update the item
        item = Tables.TableItems.getItemById(item_id)
        Tables.TableItems.updateData(item_id, new_json)
        new_json = json.read(new_json)
        self.expirePagesByTableId(item.table_id)
        return self.getTableHTML(item.table_id)

    @amiweb.expose
    def getEditView(self, item_id):
        return Tables.TableItems.getEditView(item_id)
PersonnelTables = PersonnelTables()


##
# Model
#
class PersonnelTablesModel(AmiGration):
    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.createTable("data_table",
          IntCol("id", length=11, auto_increment=True),
          StringCol("name", length=100, not_null=True),
          UniqueKey("datatable", "name"),
          PrimaryKey("id"),
          utf8=True,
        )

        self.createTable("data_tableitem",
          IntCol("id", length=11, auto_increment=True),
          IntCol("table_id", length=11),
          StringCol("json_data"),
          IntCol("item_order", length=11),
          PrimaryKey("id"),
          utf8=True,
        )

    def down(self):
        self.dropTable("datatable", "data_tableitem")


class PersonnelTablesAddHostPage(AmiGration):
    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.addColumn('data_table', IntCol("host_page", length=11))

    def down(self):
        pass

class PersonnelTablesChangeTableNames(AmiGration):
    def __init__(self):
        self.table_prefix = getConfig().TABLE_PREFIX

    def up(self):
        self.renameTable('data_table', 'plugin_personnel_table')
        self.renameTable('data_tableitem', 'plugin_personnel_item')

    def down(self):
        self.dropTable("plugin_personnel_table", "plugin_personnel_item")
