import os
import re

from amilib.amiweb import amiweb
from amilib.useful import Singleton

from skeletonz.server import getConfig, getMailManager, getFormatManager

from amilib.template import render
from skeletonz.mylib import splitter
from skeletonz.modules.plugin import GenericPlugin
from skeletonz.modules.template import ShowTemplate

PLUGINS_FOR_EXPORT = ['RegisterForm']

# hash of ident -> {mailto: 'email', show_as: ..., values: [...]}
DATA_CACHE = {}

class RegisterForm(GenericPlugin):
    NAME = "Register form"
    DESCRIPTION = "A simple framework for creating forms."
    SYNTAX = [
        {'handler': 'register_form',
            'required_arguments': {'ident': 'The unique identification',
                                   'data': {'type': 'text', 'help': 'Values separated by newline'}
                                  },
            'optional_arguments': {'show_as': 'What should the link we shown as',
                                   'mailto': 'The email that registrations should be sent to'
                                  }
        }
      ]

    def __init__(self):
        self.controller = Controller()
        getFormatManager().registerMLPlugin('register_form', self.handler)

    def addToController(self, rc):
        path = "%s/site_plugins/register_form/static/" % os.getcwd()
        rc.addStaticPath("/static_plugin/register_form/", path)
        rc.root_obj.plugin.register_form = self.controller

    def storeData(self, args):
        ident, mailto, data = args.get('register_form'), args.get('mailto', ''), args.get('data')
        global DATA_CACHE
        key = hash(ident) + hash(mailto) + hash(data)
        if not DATA_CACHE.has_key(key):
            d = {}
            d['ident'] = ident
            d['mailto'] = mailto
            d['values'] = []

            in_data = False
            for v in data.splitlines():
                if v != '':
                    if v.find(']') == 0:
                        in_data = False
                        continue
                    if in_data:
                        continue

                    if v.find('ConfirmMail=') != -1:
                        in_data = True
                        m = re.search('ConfirmMail=\[\s*((.|\n)+?)\n\]', args['data'])
                        item_data = m.group(1)
                        d['values'].append({'type': 'data', 'name': 'ConfirmMail', 'data': item_data})
                    else:
                        sp = v.split('=')
                        name = sp[0]
                        type = sp[1]
                        d['values'].append({'name': name, 'type': type})
            DATA_CACHE[key] = d
        return key

    def handler(self, args, edit_mode, page_id):
        if args.has_key('data') and args.has_key('register_form'):
            key = self.storeData(args)
            url = '%s/plugin/register_form/show?reg_nr=%s' % (getConfig().BASE_URL, key)
            args['url'] = url
            args['gb_link'] = "return GB_showFullScreen('', '%(url)s')" % args

            if not args.has_key('show_as'):
                args['show_as'] = '%(register_form)s registration' % args

            return False, '<a href="%(url)s" onclick="%(gb_link)s">%(show_as)s</a>' % args

tmpl_email = """Skeletonz has received a registration for "%s".
=============

%s
"""

class Controller(Singleton):

    def __init__(self):
        self.template = ShowTemplate("register_form")
        self.template.getHeader().append_ajs = False

    @amiweb.expose
    def show(self, reg_nr, error=None, formvars={}):
        ns = {
            'data': DATA_CACHE[int(reg_nr)],
            'reg_nr': reg_nr,
            'header': self.template.getHeader(),
            'footer': self.template.getFooter(),
            'formvars': formvars,
            'error': error
        }
        return render("site_plugins/register_form/view/show.tmpl", ns)

    def sendConfirmMail(self, formvars, uc_formvars, email_template):
        sender = getMailManager()
        subject = 'Registration for "%s"' % formvars['ident']

        message = email_template % uc_formvars
        message = message.encode('latin-1')

        try:
            sender.sendEmail(formvars['mailto'], [formvars['E-mail']], subject, message)
        except:
            return self.show(formvars['reg_nr'],
                    'Could not send a confirmation to following email: "%s"' % formvars['E-mail'], formvars)

    @amiweb.expose
    @amiweb.customHandler
    def register(self, path_info, formvars):
        def_email = getConfig().DEFAULT_EMAIL

        addr_from = [formvars['mailto']] or [def_email]
        reg_nr = formvars['reg_nr']
        subject = 'Registration for "%s"' % formvars['ident']

        uc_formvars = {}
        for k in formvars:
            uc_formvars[k] = unicode(formvars[k], 'utf8')

        cnt = []
        for pair in DATA_CACHE[int(reg_nr)]['values']:
            k = pair['name']
            if k != 'ConfirmMail':
                cnt.append("%s:\n    %s" % (k, uc_formvars[k]))
            if k == 'ConfirmMail':
                s = self.sendConfirmMail(formvars, uc_formvars, pair['data'])
                if s:
                    return s

        message = tmpl_email % (uc_formvars['ident'], "\n\n".join(cnt))
        message = message.encode('latin-1')

        sender = getMailManager()

        sender.sendEmail(def_email, addr_from, subject, message)

        ns = {
            'header': self.template.getHeader(),
            'footer': self.template.getFooter()
        }
        return render("site_plugins/register_form/view/done.tmpl", ns)
