# A simple example plugin
# RSS management code from Mark Pilgrim

# wiki syntax exmaples
# [rssreader]
# [rssreader, url=http://...] # Force source URL
# [rssreader, limit=10] # Returns only 10 first rss element
# [rssreader, url=http://feeds.feedburner.com/brainyquote/QUOTEBR, quote] # Returns only first rss element, but no table is drawn (useful for quotes)
# options can be mixed apart from 'quote' that will take the lead on 'limit'

from skeletonz.modules.plugin import GenericPlugin
from skeletonz.server import getFormatManager
from xml.dom import minidom
import urllib

PLUGINS_FOR_EXPORT = ['RSSReader']

DEFAULT_NAMESPACES = \
  (None, # RSS 0.91, 0.92, 0.93, 0.94, 2.0
  'http://purl.org/rss/1.0/', # RSS 1.0
  'http://my.netscape.com/rdf/simple/0.9/' # RSS 0.90
  )
DUBLIN_CORE = ('http://purl.org/dc/elements/1.1/',)
DEFAULT_URL = 'http://rss.slashdot.org/Slashdot/slashdot'


class RSSReader(GenericPlugin):
    NAME = "RSS reader"
    DESCRIPTION = "Read any RSS and display it on your site."
    SYNTAX = [
        {'handler': 'rssreader',
         'required_arguments': {'url': 'The URL of the feed'},
         'optional_arguments': {'as_one': 'Returns only first retrieved line'}
        }
      ]

    def __init__(self):
        getFormatManager().registerSLPlugin('rssreader', self.getRSSFeed)

    def getRSSFeed(self,args,edit_mode,page_id):
        output = []
        rssDocument = self.load(args.has_key('url') and args['url'] or DEFAULT_URL)
        for item in self.getElementsByTagName(rssDocument, 'item'):
            output.append( {'title': self.textOf(self.first(item, 'title')),
                      'link': self.textOf(self.first(item, 'link')),
                      'description': self.textOf(self.first(item, 'description')),
                      'date': self.textOf(self.first(item, 'date')),
                      'author': self.textOf(self.first(item, 'author'))
                      } )

        if args.has_key('quote'):
            return False, "<i>%(description)s</i><sup><a href='%(link)s'>*</a></sup><br>%(title)s" % output[0]
        else:
            if args.has_key('limit'): output=output[0:int(args['limit'])-1]
            return True, self.formatOutput(output)

    def load(self,rssURL):
        return minidom.parse(urllib.urlopen(rssURL))

    def getElementsByTagName(self, node, tagName, possibleNamespaces=DEFAULT_NAMESPACES):
        for namespace in possibleNamespaces:
            children = node.getElementsByTagNameNS(namespace, tagName)
            if len(children): return children
        return []

    def first(self, node, tagName, possibleNamespaces=DEFAULT_NAMESPACES):
        children = self.getElementsByTagName(node, tagName, possibleNamespaces)
        return len(children) and children[0] or None

    def textOf(self,node):
        return node and "".join([child.data for child in node.childNodes]) or ""

    def formatOutput(self, data):
        output = ['<table>']
        for row in data:
            output.append("""<tr><th>%(title)s</th><th>%(author)s</th><th>%(date)s</th><th><a onclick="return GB_showFullScreen('RSS link','%(link)s');">source</a></th></tr>""" % row)
            output.append("""<tr><td colspan=4>%s</td></tr>""" % row['description'])
        output.append('</table>')
        return "".join(output)
