from skeletonz.mylib.html_helpers import createToolTip
from skeletonz.modules.plugin import GenericPlugin
from skeletonz.user_plugins import ListOption, TextOption
from skeletonz.server import getConfig, plugin_configurator

PLUGINS_FOR_EXPORT = ['GoogieSpellConfig']

DEFAULT = "da, en"
class GoogieSpellConfig(GenericPlugin):
    NAME = "Spell check configuration"
    DESCRIPTION = "Configure spell check languages."

    """
    Languages supported
      "da": "Dansk", "de": "Deutsch", "en": "English",
      "es": "Espa&#241;ol", "fr": "Fran&#231;ais", "it": "Italiano",
      "nl": "Nederlands", "pl": "Polski", "pt": "Portugu&#234;s",
      "fi": "Suomi", "sv": "Svenska"
    """
    PLUGIN_OPTIONS = [
        TextOption(scope='googiespell_config', name="langs", text="Comma separated", default=DEFAULT),
        TextOption(scope='googiespell_config', name="default", text="Default", default="en"),
    ]

    def addToSiteEditTemplate(self, template, on_init):
        langs = plugin_configurator.get("Options_googiespell_config", "langs")
        default = plugin_configurator.get("Options_googiespell_config", "default")
        if not langs:
            langs = DEFAULT
            default = "en"

        #Build up the directory
        lang_list = langs.split(",")
        lang_list = map(lambda x: x.strip(), lang_list)

        dict_list = []
        for lang in lang_list:
            dict_list.append("'%s': GOOGIESPELL.org_lang_to_word['%s']" % (lang, lang))

        in_js = "{%s}" % ", ".join(dict_list)
        script = """
    <script>
      GOOGIE_DEFAULT_LANG = '%s';
      GOOGIESPELL.setLanguages(%s);
    </script>
        """ % (default, in_js)
        template.getHeader().appendScriptData(script)
