import types

from skeletonz.server import getFormatManager
from skeletonz.mylib.html_helpers import createToolTip
from skeletonz.modules.plugin import GenericPlugin

PLUGINS_FOR_EXPORT = ['GreyBoxImage', 'GreyBoxLink']

class GreyBoxImage(GenericPlugin):
    NAME = "GreyBox image"
    DESCRIPTION = "Open an image in a GreyBox window."
    SYNTAX = [
        {'handler': 'gb_image',
         'required_arguments': {'ident': 'The identification',
                                'link': 'The URL of the image'},
         'optional_arguments': {'caption': 'The caption of the GreyBox window'}
        }
      ]

    def __init__(self):
        getFormatManager().registerSLPlugin('gb_image', self.handlePair)

    def handlePair(self, args, edit_mode, page_id):
        if not args.has_key('caption'):
            args['caption'] = ""

        if not args.has_key('link') or args['gb_image'] == '':
            return None

        gb_image = args['gb_image']
        link = args['link']

        if type(gb_image) == types.DictType:
            view = getFormatManager().imageEditModeLink(args['gb_image'], edit_mode)
            if gb_image['plugin_type'] == 'image':
                args['gb_image'] = view
            else:
                print 'ERROR: Only images are supported in [gb_image=...]'
                return None

        if type(link) == types.DictType:
            view = getFormatManager().imageAlwaysLink(args['link'], edit_mode)
            if view:
                args['link'] = view
            else:
                print 'ERROR: Only images are supported in [gb_image..., link=...]'
                return None

        if edit_mode:
            html = []
            for a in args:
                if a not in ["gb_image", "link"]:
                    html.append("%s = %s" % (a, args[a]))

            args['args'] = createToolTip("<br />".join(html), "args...")
            return False, """[<span class="CMS_combinator">gb_image</span>=%(gb_image)s, link=%(link)s, %(args)s]""" % args
        else:
            return False, """<a href="%(link)s" onclick="return GB_showImage('%(caption)s', this.href)">%(gb_image)s</a>""" % args


class GreyBoxLink(GenericPlugin):
    NAME = "GreyBox link"
    DESCRIPTION = "Open a URL in a GreyBox window."
    SYNTAX = [
        {'handler': 'gb_link',
         'required_arguments': {'ident': 'The identification',
                                'link': 'The URL of the page'},
         'optional_arguments': {'caption': 'The caption of the GreyBox window',
                                'width': 'The width of the GreyBox window<br />Should be an integer',
                                'height': 'The height of the GreyBox window<br />Should be an integer',
                                'fullscreen': {'type': 'option', 'help': 'Should the width and height be full screen?'}}
        }
      ]

    def __init__(self):
        getFormatManager().registerSLPlugin('gb_link', self.handlePair)

    def handlePair(self, args, edit_mode, page_id):
        if not args.has_key('caption'):
            args['caption'] = ""

        if not args.has_key('link') or args['gb_link'] == '':
            return None

        if not args.has_key('fullscreen'):
            if not args.has_key('width'):
                args['width'] = 500
            if not args.has_key('height'):
                args['height'] = 500
        else:
            args['fullscreen'] = 'on'

        gb_link = args['gb_link']
        link = args['link']

        if type(gb_link) == types.DictType:
            view = getFormatManager().imageEditModeLink(args['gb_link'], edit_mode)
            if view:
                args['gb_link'] = view
            else:
                print 'ERROR: Only images are supported in [gb_link=...]'
                return None

        if type(link) == types.DictType:
            view = getFormatManager().imageAlwaysLink(args['link'], edit_mode)
            if view:
                args['link'] = view
            else:
                print 'ERROR: Only images are supported in [gb_link=..., link=...]'
                return None

        if edit_mode:
            html = []
            for a in args:
                if a != "gb_link":
                    html.append("%s = %s" % (a, args[a]))
            args['link'] = createToolTip("<br />".join(html), "args...")

            return False, """[<span class="CMS_combinator">gb_link</span>=%(gb_link)s, %(link)s]""" % (args)
        else:
            if args.has_key('fullscreen'):
                return False, """<a href="%(link)s" onclick="return GB_showFullScreen('%(caption)s', '%(link)s')">%(gb_link)s</a>""" % args
            else:
                return False, """<a href="%(link)s" onclick="return GB_show('%(caption)s', '%(link)s', %(height)s, %(width)s)">%(gb_link)s</a>""" % args
