import re

from skeletonz.modules.plugin import GenericPlugin
from skeletonz.server import getFormatManager

from pygments import highlight
from pygments.formatters import get_formatter_by_name
from pygments.lexers import get_lexer_by_name

PLUGINS_FOR_EXPORT = ['CodeHighlighter']

LANGS = [
    ('C', 'c'),
    ('C++', 'cpp'),
    ('CSS', 'css'),
    ('Diff', 'diff'),
    ('HTML', 'html'),
    ('Java', 'java'),
    ('JavaScript', 'js'),
    ('PHP', 'php'),
    ('Perl', 'pl'),
    ('Python', 'py'),
    ('Ruby', 'rb'),
    ('Tex', 'tex'),
    ('XML', 'xml')
]

class CodeHighlighter(GenericPlugin):
    NAME = "Code highlighter"
    DESCRIPTION = "Can syntax highlight your code in a lot of languages"
    SYNTAX = [
        {'handler': 'hlcode',
         'required_arguments':
            {'ident': {'type': 'list', 'help': 'Syntax langauges',
                      'dict': LANGS, 'default': 'HTML'},
             'data': {'type': 'text', 'help': 'The code'},
         },

         'optional_arguments': {
         }
        }
      ]

    def __init__(self):
        self.data = None
        getFormatManager().registerMLPlugin('hlcode', self.handleFilter)

    def add(self, template, on_init):
        file = "skeletonz/plugins/hlcode/pygments_style.css"
        if on_init:
            template.getHeader().appendStaticStyleData(open(file).read())

    def addToSiteEditTemplate(self, template, on_init):
        self.add(template, on_init)

    def addToSiteTemplate(self, template, on_init):
        self.add(template, on_init)

    def returnContentFilters(self):
        return [('hlcode', self.handleFilter)]

    def handleFilter(self, args, edit_mode, page):
        if args.has_key("data"):
            data = args['data'].strip()
            lexer = get_lexer_by_name(args.get('hlcode') or args.get('language'))
            formatter = get_formatter_by_name("html")
            if lexer:
                return True, highlight(data, lexer, formatter)
