import os

from amilib.amiweb import amiweb

from amilib.template import render
from skeletonz.modules.template import PluginTemplate
from skeletonz.modules.plugin import GenericPlugin
from skeletonz.server import getConfig, getFormatManager

PLUGINS_FOR_EXPORT = ['SyntaxHelpPlugin']

class SyntaxHelpPlugin(GenericPlugin):
    NAME = "Syntax help"
    DESCRIPTION = "Display Skeletonz syntax help on your own pages."
    SYNTAX = [
        {'handler': 'syntax_help',
         'required_arguments': {'ident': "Can be text, internal or external. <br /> Depends on what help you want to show."},
         'optional_arguments': {}
        }
      ]

    def __init__(self):
        getFormatManager().registerSLPlugin('syntax_help', self.handleFilter)

    def addToController(self, rc):
        rc.root_obj.syntax_help = SyntaxHelpController()

    def add(self, template, on_init):
        if on_init:
            header = template.getHeader()
            header.appendStyle("%s/skeletonz/plugins/syntax_help/static/syntax_help.css" % os.getcwd())

    def addToSiteEditTemplate(self, template, on_init):
        self.add(template, on_init)

    def addToSiteTemplate(self, template, on_init):
        self.add(template, on_init)

    def handleFilter(self, args, edit_mode, page):
        page = args.get('syntax_help', None)
        syntax_help = SyntaxHelpController()
        content = ''

        if not page:
            return None

        elif page == "text":
            content = syntax_help.getTextHelp()
        elif page == "internal":
            content = syntax_help.getInternalHelp()
        elif page == "external":
            content = syntax_help.getExternalHelp()

        return True, '<div class="syntax_help">%s</div>' % content


class SyntaxHelpController:

    def __init__(self):
        self.p = 'skeletonz/plugins/syntax_help/view'
        self.template = PluginTemplate("SyntaxHelp")

    @amiweb.expose
    def index(self, page="text"):
        fn_cnt = self.getTextHelp

        if page == "internal":
            fn_cnt = self.getInternalHelp
        elif page == "external":
            fn_cnt = self.getExternalHelp

        header = self.template.getHeader()
        header.appendStyle("%s/skeletonz/plugins/syntax_help/static/syntax_help.css" % os.getcwd())

        ns = {'template': self.template,
              'getContent': fn_cnt,
              'current': page}
        return render("%s/index.tmpl" % self.p, ns)

    @amiweb.expose
    def getTextHelp(self):
        return render("%s/text_help.tmpl" % self.p, {}, base="%s/functions.tmpl" % self.p)

    @amiweb.expose
    def getInternalHelp(self):
        return render("%s/internal_help.tmpl" % self.p, {}, base="%s/functions.tmpl" % self.p)

    @amiweb.expose
    def getExternalHelp(self):
        return render("%s/external_help.tmpl" % self.p, {}, base="%s/functions.tmpl" % self.p)
