import os, sys
sys.path.insert(0, os.path.abspath("."))
sys.path.insert(0, os.path.abspath("amilib"))

import py.test
from amilib.amiweb.amigration import *
from amilib.amiweb import amiweb
from amilib.amiweb.amiweb import db

import general_config

amiweb.setUpDatabase(general_config.DB_HOST, general_config.DB_USER, general_config.DB_PASSWORD, 'amigration', '')
amiweb.setUpConnection()

class News_Initial(AmiGration):
    def __init__(self):
        self.table_prefix = "hello_"

    def up(self):
        self.createTable("news",
          IntCol("id", auto_increment=True),
          StringCol("headline", length=100),
          StringCol("content"),
          StringCol("url", length=50),
          UniqueKey("name", "headline", "url"),
          PrimaryKey("id"),
          utf8=True,
          ignore_if_created=True
        )
        self.renameTable("news", "news_also")
        self.renameTable("news_also", "news")

    def down(self):
        self.dropTable("news")

class News_AddColumns(AmiGration):
    def __init__(self):
        self.table_prefix = "hello_"

    def up(self):
        self.addColumn("news", StringCol("author", length=50), IntCol("content_len", length=5))
        self.addColumn("news", StringCol("cowmad", length=50, not_null=True))
        self.changeColumn("news", StringCol("author", length=66))

    def down(self):
        self.removeColumn("news", "author", "cowmad", "content_len")

class News_AddKey(AmiGration):
    def __init__(self):
        self.table_prefix = "hello_"

    def up(self):
        self.addColumn("news", IndexKey("cowmad", "cowmad", "author"))

    def down(self):
        self.removeKey("news", "cowmad")


def test_amigration():
    cnf = amiweb.Configurator("test")
    mi_control = AMigrationControl("news_plugin", [News_Initial, News_AddColumns, News_AddKey], cnf)
    mi_control.upgradeToLatest()

    db().insert("hello_news", author="Hallo")
    a_news = db().select("hello_news", as_one=True)
    assert a_news.has_key("author") == True

    mi_control.downgradeTo("News_Initial")
    a_news = db().select("hello_news", as_one=True)
    assert a_news.has_key("author") == False

    mi_control.revision_list = [News_Initial, News_AddColumns, News_AddKey]
    mi_control.upgradeToLatest()

    mi_control.downgradeTo("News_AddColumns")
    cnf.deleteAll()
    #mi_control.upgradeToLatest()

    #mi_control.downgradeTo(Not_Created)
