import sys, os
sys.path.insert(0, os.path.abspath(""))
sys.path.insert(0, os.path.abspath("lib"))
sys.path.insert(0, os.path.abspath("skeletonz"))

from skeletonz.model.UserModel import Users, Groups
from skeletonz.model.CMSModel import MenuItems, Menus, Pages, PageLinks

import general_config

from amiweb import amiweb
amiweb.setUpDatabase(general_config.DB_HOST, general_config.DB_USER, general_config.DB_PASSWORD, general_config.DB_DATABASE, general_config.TABLE_PREFIX)

amiweb.setUpConnection()

#Test users
def test_users():
    admin = Users.addUser("admin", "blah")
    assert admin == None

    amixo = Users.addUser("amixo", "juuhu")
    assert amixo != None
    assert amixo.username == "amixo"

    assert amixo.toJSON() != None

    all_users = Users.getAllUsers()
    assert all_users != None

def test_groups():
    rider = Groups.addGroup("riders")
    assert rider != None

def test_user_n_group():
    assert Groups.isInGroup("riders", "amixo") == False
    g = Groups.getGroupByName("riders")
    u = Users.getUserByUsername("amixo")
    Groups.changeUserGroup(g.id, u.id)
    assert Groups.isInGroup("riders", "amixo") == True

#Test amidb
def test_json():
    class DecoObj:
        def toJSON(self):
            return "{'test': 'just a test'}"

    json_normal = amiweb.db().select("menu_item", id=1, as_one=True).toJSON()
    json_custom = amiweb.db().select("menu_item", id=1, as_one=True, obj_deco=DecoObj).toJSON()

    assert json_normal != None
    assert json_custom != None
    assert json_normal != json_custom

#CMSModel
def test_menu_items():
    m_n_item = MenuItems.addItem("Juuuhuu", 1, 1)
    m_en_item = MenuItems.addItemExternal("Google", "http://www.google.com", 1, 1)
    assert m_n_item != None
    assert m_en_item != None

    m_item = MenuItems.getItemById(1)
    assert m_item.toJSON() != None
    assert MenuItems.getItemByPageId(1) != None

    #Update
    MenuItems.editItem(m_n_item.id, "Juuhu Booh")
    assert MenuItems.getItemById(m_n_item.id).name == "Juuhu Booh"

    #Try updating external
    MenuItems.editItem(m_en_item.id, "Google 2")
    assert MenuItems.getItemById(m_en_item.id).name == "Google 2"
    assert MenuItems.getItemById(m_en_item.id).page_id == None

    #Update order
    MenuItems.setNewOrder(m_en_item.id, 10)
    assert MenuItems.getItemById(m_en_item.id).m_order == 10

#Menu testing
def test_menu():
    new_menu1 = Menus.insertMenu("Joobla")
    new_menu2 = Menus.insertMenu("Joobla")
    assert new_menu1.id == new_menu2.id != None

    MenuItems.addItem("Black", 1, new_menu1.id)

    #Test get all items and toJSON
    m1 = Menus.getMenuByName("Joobla")
    assert m1 != None
    assert len(m1.getAllItems()) == 1
    assert m1.getAllItems()[0].name == "Black"
    assert m1.toJSON() != None
    assert m1.getAllItems()[0].toJSON() != None

    #Test the other
    assert len(m1.getAllPageItems()) == 1
    assert m1.getTopItem().name == "Black"

#Page testing
def test_page():
    assert len(Pages.getAllPages()) > 0
    assert Pages.getFrontPage().name == "About"
    assert Pages.getPage("About", None).name == "About"

    page = Pages.insertPage("Juhu", "Cow er1er lort", 1)
    assert page != None
    assert page.id != None

    assert Pages.getPage("Juhu", None).id == Pages.getPageById(page.id).id

    assert Pages.getAllPagesWhereTextIsUsed("er1er")[0].id == page.id
    Pages.resetPremissions(page.id)
    assert Pages.getPremission(page.id) == ("Every user", None)

def test_page_parent():
    #Create a page that has a parent
    p_parent = Pages.insertPage("Blah parent", "Dett er bare en test", 1)
    p_child = Pages.insertPageFromWiki(p_parent.id, "Blah child")
    p_child_child = Pages.insertPageFromWiki(p_child.id, "Blah child child")
    assert p_parent.getChildren() != None
    assert p_parent.getChildren()[0].id == p_child.id
    assert p_child.getChildren()[0].id == p_child_child.id

    #Delete the parent -> the children should also be deleted
    Pages.deletePageById(p_parent.id)
    assert Pages.getPageById(p_parent.id) == None
    assert Pages.getPageById(p_child.id) == None
    assert Pages.getPageById(p_child_child.id) == None

def test_page_alter():
    p = Pages.insertPage("TestyBesty", "Dett er bare en test", 1)
    #Set title
    Pages.setPageTitle(p.id, "JahooSir")
    assert Pages.getPageById(p.id).title == "JahooSir"

    #Set menu
    Pages.setPageMenu(p.id, 5)
    assert Pages.getPageById(p.id).menu_id == 5

    #Set content
    Pages.setPageContent(p.id, "New content")
    p = Pages.getPageById(p.id)
    assert p.getContent() == "<p>New content</p>"
    assert p.getContentEdit() == p.getContent()

    Pages.deletePageById(p.id)

def test_page_links():
    p1 = Pages.insertPage("Page1", "Blah blah", 1)
    p2 = Pages.insertPage("Page2", "Blah blah", 1)

    pa_link = PageLinks.createPageLink("Link to page 1")
    assert PageLinks.getPageLinkByIdent("Link to page 1") != None

    PageLinks.set(pa_link.id, p1.id)
    assert PageLinks.getPageLinkByIdent("Link to page 1").page_id == p1.id

    PageLinks.set(pa_link.id, p2.id)
    assert PageLinks.getPageLinkByIdent("Link to page 1").page_id == p2.id

    PageLinks.set(pa_link.id, 0)
    assert PageLinks.getPageLinkByIdent("Link to page 1").page_id == None

    PageLinks.deletePageLinkById(pa_link.id)

    Pages.deletePageById(p1.id)
    Pages.deletePageById(p2.id)

def test_delete():
    #Delete all the menu items that we created
    map(lambda o: MenuItems.deleteOnlyItem(o.id), MenuItems.getItemByName("Juuuhuu"))
    map(lambda o: MenuItems.deleteOnlyItem(o.id), MenuItems.getItemByName("Google"))
    map(lambda o: MenuItems.deleteOnlyItem(o.id), MenuItems.getItemByName("Google 2"))
    map(lambda o: MenuItems.deleteOnlyItem(o.id), MenuItems.getItemByName("Juuhu Booh"))

    assert len(MenuItems.getItemByName("Juuuhuu")) == 0
    assert len(MenuItems.getItemByName("Juuhu Booh")) == 0
    assert len(MenuItems.getItemByName("Google")) == 0
    assert len(MenuItems.getItemByName("Google 2")) == 0

    #Delete all the menus and menu items
    joobla = Menus.getMenuByName("Joobla")
    assert joobla != None
    Menus.deleteMenu(joobla.id)

    assert Menus.getMenuByName("Joobla") == None
    assert len(MenuItems.getItemsByMenuId(joobla.id)) == 0

    #Delete the Juhu page
    p1 = Pages.getPage("Juhu", None)
    p2 = Pages.getPage("Juuuhuu", None)
    p3 = Pages.getPage("Black", None)
    Pages.deletePageById(p1.id)
    Pages.deletePageById(p2.id)
    Pages.deletePageById(p3.id)

def teardown_module(module):
    #Delete test user amixo
    amixo = Users.getUserByUsername("amixo")
    if amixo:
        Users.deleteUser(amixo.id)

    #Delete test group riders
    group = Groups.getGroupByName("riders")
    Groups.deleteGroup(group.id)
